#
#   setup-dbg.py
#   Python distutil package setup, with debug features enabled
#
#   Copyright 2009      David Sommerseth <davids@redhat.com>
#   Copyright 2007-2009 Nima Talebi <nima@autonomy.net.au>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
#   For the avoidance of doubt the "preferred form" of this code is one which
#   is in an open unpatent encumbered format. Where cryptographic key signing
#   forms part of the process of creating an executable the information
#   including keys needed to generate an equivalently functional executable
#   are deemed to be part of the source code.
#

from distutils.core import setup, Extension
from setup_common import *

#
#  Some default values
#
incdir = []
libdir = []
libs = []

# Get libxml2 info
libxml2_include(incdir)
libxml2_lib(libdir, libs)

# misc info
dmidec_version = get_version()
macros = get_macros()

#
#  Python setup
#

setup(
  name = "python-dmidecode-dbg",
  version = dmidec_version,
  license='GPL-2',
  description = "Python extension module for dmidecode",
  author = "Nima Talebi & David Sommerseth",
  author_email = "nima@it.net.au, davids@redhat.com",
  url = "http://projects.autonomy.net.au/python-dmidecode/",
  data_files = [ ('share/python-dmidecode-dbg', ['src/pymap.xml']) ],
  ext_modules = [
    Extension(
      "dmidecodemod",
      sources      = [
        "src/dmidecodemodule.c",
        "src/util.c",
        "src/dmioem.c",
        "src/dmidecode.c",
        "src/dmixml.c",
        "src/dmierror.c",
        "src/dmilog.c",
        "src/xmlpythonizer.c",
        "src/efi.c",
        "src/dmidump.c"
      ],
      include_dirs = incdir,
      library_dirs = libdir,
      libraries    = libs,
      undef_macros = [ "NDEBUG" ],
      define_macros = macros
    )
  ],
  py_modules = [ "dmidecode-dbg" ]
)



#!/usr/bin/env python
#
#   Examples which makes use of the different python-dmidecode features
#   This script should be run as root, or else expect permission warnings
#
#   Copyright 2008-2009 Nima Talebi <nima@autonomy.net.au>
#   Copyright 2010      David Sommerseth <davids@redhat.com>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
#   For the avoidance of doubt the "preferred form" of this code is one which
#   is in an open unpatent encumbered format. Where cryptographic key signing
#   forms part of the process of creating an executable the information
#   including keys needed to generate an equivalently functional executable
#   are deemed to be part of the source code.
#

import dmidecode
import sys, os
from pprint import pprint

def print_warnings():
        "Simple function, dumping out warnings with a prefix if warnings are found and clearing warning buffer"
        warn = dmidecode.get_warnings()
        if warn:
              print("### WARNING: %s" % warn)
              dmidecode.clear_warnings()


# Check if running as root .... provide a warning if not
root_user = (os.getuid() == 0 and True or False)
if not root_user:
        print("####")
        print("####  NOT RUNNING AS ROOT")
        print("####")
        print("#### The first run must always be done as root for this example to work.")
        print("#### When not run as root, quite some permission errors might appear")
        print("####")
        print("#### If this script is first run as root, it should be possible to run this script")
        print("#### as an unprivileged user afterwards, with less warnings.")
        print("####")
        print()
        print()


#. Test for presence of important functions using /dev/mem...  Using the legacy API
#. This does not print any decoded info.  If the call fails, either a warning will
#. be issued or an exception will be raised.  This test is now only used to check
#. for presence of the legacy API, which "under the hood" uses
#. dmidecode.QuerySection(name), where name can be 'bios', 'system', etc.
if root_user:
        print("*** bios ***\n");      dmidecode.bios()
        print_warnings()
        print("*** system ***\n");    dmidecode.system()
        print_warnings()
        print("*** baseboard ***\n"); dmidecode.baseboard()
        print_warnings()
        print("*** chassis ***\n");   dmidecode.chassis()
        print_warnings()
        print("*** processor ***\n"); dmidecode.processor()
        print_warnings()
        print("*** memory ***\n");    dmidecode.memory()
        print_warnings()
        print("*** cache ***\n");     dmidecode.cache()
        print_warnings()
        print("*** connector ***\n"); dmidecode.connector()
        print_warnings()
        print("*** slot ***\n");      dmidecode.slot()
        print_warnings()


#. Now test get/set of memory device file...
print("*** get_dev()")
print(dmidecode.get_dev())
print_warnings()
print("*** set_dev('dmidata.dump')")
print(dmidecode.set_dev("dmidata.dump"));
print_warnings()
print("*** get_dev()")
print(dmidecode.get_dev())
print_warnings()

#. Test taking a dump...
if root_user:
        print("*** Dumping DMI data to dump file")
        print(dmidecode.dump())
        print_warnings()

#. Test reading the dump...  Using the preferred API
print("*** bios ***\n");      pprint(dmidecode.QuerySection('bios'))
print_warnings()
print("*** system ***\n");    pprint(dmidecode.QuerySection('system'))
print_warnings()
print("*** baseboard ***\n"); pprint(dmidecode.QuerySection('baseboard'))
print_warnings()
print("*** chassis ***\n");   pprint(dmidecode.QuerySection('chassis'))
print_warnings()
print("*** processor ***\n"); pprint(dmidecode.QuerySection('processor'))
print_warnings()
print("*** memory ***\n");    pprint(dmidecode.QuerySection('memory'))
print_warnings()
print("*** cache ***\n");     pprint(dmidecode.QuerySection('cache'))
print_warnings()
print("*** connector ***\n"); pprint(dmidecode.QuerySection('connector'))
print_warnings()
print("*** slot ***\n");      pprint(dmidecode.QuerySection('slot'))
print_warnings()

print("*** Extracting memory information")
for v in dmidecode.memory().values():
  if type(v) == dict and v['dmi_type'] == 17:
    pprint(v['data']['Size']),

print("*** Querying for DMI type 3 and 7")
pprint(dmidecode.type(3))        # <-- Legacy API
pprint(dmidecode.QueryTypeId(7)) # <-- preferred API
print_warnings()

print("*** Querying for the BIOS section")
pprint(dmidecode.QuerySection('bios'))
print_warnings()

#
# Test XML stuff
#
print()
print()
print()
print("---------------------------------------")
print("*** *** *** Testing XML API *** *** ***")
print("---------------------------------------")
print()
print()
dmixml = dmidecode.dmidecodeXML()

# Fetch all DMI data into a libxml2.xmlDoc object
print("*** Getting all DMI data into a XML document variable")
dmixml.SetResultType(dmidecode.DMIXML_DOC)  # Valid values: dmidecode.DMIXML_DOC, dmidecode.DMIXML_NODE
xmldoc = dmixml.QuerySection('all')

# Dump the XML to dmidump.xml - formated in UTF-8 decoding
print("*** Dumping XML document to dmidump.xml")
xmldoc.saveFormatFileEnc('dmidump.xml','UTF-8',1)

# Do some XPath queries on the XML document
print("*** Doing some XPath queries against the XML document")
dmixp = xmldoc.xpathNewContext()

# What to look for - XPath expressions
keys = ['/dmidecode/SystemInfo/Manufacturer',
	'/dmidecode/SystemInfo/ProductName',
	'/dmidecode/SystemInfo/SerialNumber',
	'/dmidecode/SystemInfo/SystemUUID']

# Extract data and print it
for k in keys:
	data = dmixp.xpathEval(k)
	for d in data:
		print("%s: %s" % (k, d.get_content()))

del dmixp
del xmldoc

# Query for only a particular DMI TypeID - 0x04 - Processor
print("*** Quering for Type ID 0x04 - Processor - dumping XML document to stdout")
dmixml.QueryTypeId(0x04).saveFormatFileEnc('-','UTF-8',1)
print_warnings()



#
#   setup-common.py
#   Helper functions for retrieving libxml2 arguments needed for compilation
#   and other functions which is used in both setup.py and setup-dbg.py
#
#   Copyright 2009      David Sommerseth <davids@redhat.com>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
#   For the avoidance of doubt the "preferred form" of this code is one which
#   is in an open unpatent encumbered format. Where cryptographic key signing
#   forms part of the process of creating an executable the information
#   including keys needed to generate an equivalently functional executable
#   are deemed to be part of the source code.
#

import subprocess, sys
if sys.version_info[0] < 3:
    import commands as subprocess
from os import path as os_path
from distutils.sysconfig import get_python_lib

# libxml2 - C flags
def libxml2_include(incdir):
    (res, libxml2_cflags) = subprocess.getstatusoutput("xml2-config --cflags")
    if res != 0:
        print("Could not build python-dmidecode.")
        print("Could not run xml2-config, is libxml2 installed?")
        print("Also the development libraries?")
        sys.exit(1)

    # Parse the xml2-config --cflags response
    for l in libxml2_cflags.split(" "):
        if l.find('-I') == 0:
            incdir.append(l.replace("-I", "", 1))



# libxml2 - library flags
def libxml2_lib(libdir, libs):
    libdir.append(get_python_lib(1))
    if os_path.exists("/etc/debian_version"): #. XXX: Debian Workaround...
        libdir.append("/usr/lib/pymodules/python%d.%d"%sys.version_info[0:2])

    (res, libxml2_libs) = subprocess.getstatusoutput("xml2-config --libs")
    if res != 0:
        print("Could not build python-dmidecode.")
        print("Could not run xml2-config, is libxml2 installed?")
        print("Also the development libraries?")
        sys.exit(1)

    # Parse the xml2-config --libs response
    for l in libxml2_libs.split(" "):
        if l.find('-L') == 0:
            libdir.append(l.replace("-L", "", 1))
        elif l.find('-l') == 0:
            libs.append(l.replace("-l", "", 1))

    # this library is not reported and we need it anyway
    libs.append('xml2mod')



# Get version from src/version.h
def get_version():
    version = "0.0.0"
    try:
        f = open("src/version.h")
    except:
        f = open("version.h")

    try:
        for line in f:
            part = line.split(" ")
            if part[0] == "#define":
                if part[1] == "VERSION":
                    version = part[2].strip().strip('"')
                    break
    finally:
        f.close()

    return version

def get_macros():
    "Sets macros which is relevant for all setup*.py files"

    macros = []
    if sys.byteorder == 'big':
        macros.append(("ALIGNMENT_WORKAROUND", None))
    return macros

